----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    13:25:55 06/30/2014 
-- Design Name: 
-- Module Name:    GTOCORE - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;

use WORK.GTOPackage.ALL;

entity GTOCORE is
    Generic(INITVAL : gtovalarray);
    Port ( CLOCK : in  STD_LOGIC;
           XRX : in  STD_LOGIC;
           XTX : out  STD_LOGIC;
           ECS : out  STD_LOGIC;
           EDI : in  STD_LOGIC;
           EDO : out  STD_LOGIC;
           ECK : out  STD_LOGIC;
           rst : out  STD_LOGIC;
           active  : out  STD_LOGIC;
           ucomf   : out STD_LOGIC;
           utxf    : in  STD_LOGIC;
           uaddr   : out  STD_LOGIC_VECTOR (5 downto 0);
           urxdata : out  STD_LOGIC_VECTOR (7 downto 0);
           val     : out gtovalarray;
           userval : in gtovalarray);
end GTOCORE;

architecture Behavioral of GTOCORE is
	COMPONENT RS232RX
  GENERIC (MARGIN : integer;
           SKIP : integer);
	PORT(
		clk : IN std_logic;
		rx : IN std_logic;
		rxe : IN std_logic;          
		rxf : OUT std_logic;
		data : OUT std_logic_vector(7 downto 0)
		);
	END COMPONENT;

	COMPONENT RS232TX
  GENERIC (SKIP : integer);
	PORT(
		clk : IN std_logic;
		txf : IN std_logic;
		data : IN std_logic_vector(7 downto 0);          
		txe : OUT std_logic;
		tx : OUT std_logic
		);
	END COMPONENT;

	COMPONENT DIVIDER
  GENERIC (DIV : integer);
	PORT(
		S : IN std_logic;          
		Q : OUT std_logic
		);
	END COMPONENT;

	COMPONENT LEDDRV
	PORT(
		clk : IN std_logic;
		s : IN std_logic;          
		q : OUT std_logic
		);
	END COMPONENT;

	COMPONENT GTOCOM
	PORT(
		clk : IN std_logic;
		rxf : IN std_logic;
		rdata : IN std_logic_vector(7 downto 0);          
		txf : OUT std_logic;
		addr : OUT std_logic_vector(7 downto 0);
		data : OUT std_logic_vector(7 downto 0)
		);
	END COMPONENT;

	COMPONENT GTOVAL
  GENERIC (
    INIT : std_logic_vector(7 downto 0);
    ADDR : integer);
	PORT(
		s : IN std_logic;
		rst : IN std_logic;
		vaddr : IN std_logic_vector(5 downto 0);
		din : IN std_logic_vector(7 downto 0);          
		dout : OUT std_logic_vector(7 downto 0)
		);
	END COMPONENT;

    COMPONENT GTOEPCOM
    PORT(
         clk : IN  std_logic;
         wrf : IN  std_logic;
         rdf : IN  std_logic;
         wrval : IN  gtovalarray;
         rdwork : out std_logic;
         rdvalf : out std_logic;
         rdvaladdr : out std_logic_vector(5 downto 0);
         rdval : OUT std_logic_vector(7 downto 0);
         EDI : IN  std_logic;
         ECS : OUT  std_logic;
         EDO : OUT  std_logic
        );
    END COMPONENT;

signal xtxe, xtxf : std_logic := '0';
signal epclk, sclk, act, comf, rxf, wvalf, irst, readf, epwork, mwvalf, ieprdf, iepwrf : std_logic := '0';
signal xtxdata, data, addr, rxdata, mdata : std_logic_vector(7 downto 0) := x"00";
signal com : std_logic_vector(1 downto 0);
signal vaddr, mvaddr : std_logic_vector(5 downto 0);
signal ival, xval : gtovalarray;
signal epwrf, eprdf, epvalf : std_logic := '0';
signal epvaladdr : std_logic_vector(5 downto 0);
signal epval : std_logic_vector(7 downto 0);
signal epcnt : std_logic_vector(7 downto 0) := x"00";
signal initep : std_logic := '1';

type TXSTATE is (TXNOOP, TXVAL, TXLAST);
signal txstat : TXSTATE := TXNOOP;
type TXMODESTATE is (BUILTIN, USER);
signal txmode : TXMODESTATE := BUILTIN;

begin

vaddr <= addr(5 downto 0);
com <= addr(7 downto 6);

rst <= irst;
irst    <= comf  when com  = "01" and vaddr = "000001" else '0';
readf   <= comf  when com  = "01" and vaddr = "000000" else '0';
ieprdf   <= comf  when com = "01" and vaddr = "000010" else '0';
iepwrf   <= comf  when com = "01" and vaddr = "100000" else '0';
wvalf   <= comf  when com  = "10" else '0';
--ucomf   <= comf  when com  = "00" else '0';
uaddr   <= vaddr when com  = "00" else "000000";
urxdata <= data  when com  = "00" else x"00";

  process(sclk)
  begin
  if(sclk'event and sclk = '1') then
    if(comf = '1') then
      if(com = "00") then
        ucomf <= '1';
      else
        ucomf <= '0';
      end if;
    else
      ucomf <= '0';
    end if;
  end if;
  end process;

mwvalf <= wvalf when epwork = '0' else epvalf;
mdata <= data when epwork = '0' else epval;
mvaddr <= vaddr when epwork = '0' else epvaladdr;

act <= rxf or epwork;
val <= ival;
xval <= ival when txmode = BUILTIN else userval;

	LEDDRVRX: LEDDRV PORT MAP(
		clk => CLOCK,
		s => act,
		q => active
	);
  
	Inst_RS232RX: RS232RX
  GENERIC MAP(
    MARGIN => 20,
    SKIP => 54)
  PORT MAP(
		clk => sclk,
		rx => XRX,
		rxe => '1',
		rxf => rxf,
		data => rxdata
	);

	Inst_RS232TX: RS232TX
  GENERIC MAP(
    SKIP => 54)
  PORT MAP(
    clk => sclk,
		tx => XTX,
		txe => xtxe,
		txf => xtxf,
		data => xtxdata
	);

	DIVIDERA: DIVIDER
  GENERIC MAP(
    DIV => 4)
  PORT MAP(
		S => CLOCK,
		Q => sclk
	);

	DIVIDERB: DIVIDER
  GENERIC MAP(
    DIV => 100)
  PORT MAP(
		S => CLOCK,
		Q => epclk
	);


	Inst_GTOCOM: GTOCOM PORT MAP(
		clk => sclk,
		rxf => rxf,
		rdata => rxdata,
		txf => comf,
		addr => addr,
		data => data
	);


gtovalgen:
for i in 0 to 63 generate
	GTOVALInst: GTOVAL
  GENERIC MAP(
    INIT => initval(i),
    ADDR => i)
  PORT MAP(
		s => mwvalf,
		rst => irst,
		vaddr => mvaddr,
		din => mdata,
		dout => ival(i)
	);
end generate;

   GTOEPCOMInst: GTOEPCOM PORT MAP (
     clk => epclk,
     wrf => epwrf,
     rdf => eprdf,
     wrval => ival,
     rdwork => epwork,
     rdval => epval,
     rdvalf => epvalf,
     rdvaladdr => epvaladdr,
     EDI => EDI,
     ECS => ECS,
     EDO => EDO
    );


ECK <= epclk;


-- Readout GTOVAL
  process(sclk)
  variable t : integer range 0 to 63;
  begin
  if(sclk'event and sclk = '0') then
  case txstat is
  when TXNOOP =>
    xtxf <= '0';
    t := 0;
    xtxdata <= x"00";
    if(readf = '1' or utxf = '1') then
      txstat <= TXVAL;
      if(utxf = '1') then
        txmode <= USER;
      else
        txmode <= BUILTIN;
      end if;
    else
      txstat <= TXNOOP;
    end if;
  when TXVAL =>
    if(xtxe = '1') then
      xtxf <= '1';
      xtxdata <= xval(t);
      if(t = 63) then
        txstat <= TXLAST;
      else
        t := t + 1;
      end if;
    else
      xtxf <= '0';
      xtxdata <= xtxdata;
      txstat <= TXVAL;
      t := t;
    end if;
  when TXLAST =>
    xtxdata <= xtxdata;
    if(xtxe = '1') then
      txstat <= TXNOOP;
      txmode <= BUILTIN;
      t := 0;
    else
      txstat <= TXLAST;
    end if;
  end case;
  end if;
  end process;

-- to generate ep rd/wr flag
  process(sclk)
  begin
  if(sclk'event and sclk = '0') then
    if(epcnt = x"00") then
      if(ieprdf = '1' or initep = '1') then
        eprdf <= '1';
        epwrf <= '0';
        epcnt <= x"01";
      elsif(iepwrf = '1') then
        eprdf <= '0';
        epwrf <= '1';
        epcnt <= x"01";
      else
        eprdf <= '0';
        epwrf <= '0';
        epcnt <= x"00";
      end if;
    else
      epcnt <= epcnt + '1';
      if(epcnt = 100) then
        epcnt <= x"00";
        eprdf <= '0';
        epwrf <= '0';
      end if;
    end if;
  end if;
  end process;

  FDCInst: FDCE
  generic map(
    INIT => '1')
  PORT MAP(
    Q => initep,
    C => '0',
    CE => '0',
    CLR => eprdf,
    D => '0'
  );


end Behavioral;

