----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    16:02:44 08/29/2012 
-- Design Name: 
-- Module Name:    EPCOM - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
library WORK;
use WORK.GTOPackage.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;

entity GTOEPCOM is
    Port ( clk : in  STD_LOGIC;
           wrf : in  STD_LOGIC;
           rdf : in  STD_LOGIC;
           wrval : in gtovalarray;
           rdvaladdr : out STD_LOGIC_VECTOR(5 downto 0);
           rdval : out STD_LOGIC_VECTOR(7 downto 0);
           rdvalf : out STD_LOGIC;
           rdwork : out STD_LOGIC;
           EDI : in STD_LOGIC;
           ECS : out STD_LOGIC;
           EDO : out STD_LOGIC);
end GTOEPCOM;

architecture Behavioral of GTOEPCOM is

-- ADDRESS List of 32bytes bank EEPROM
type ADDRLIST is array(0 to 1) of std_logic_vector(0 to 15);
constant ADDR : ADDRLIST := (x"0000", x"0020");

-- EEPROM Register
constant RDSR : std_logic_vector(7 downto 0) := "10100000";
constant WRSR : std_logic_vector(7 downto 0) := "10000000";
constant WREN : std_logic_vector(7 downto 0) := "01100000";
constant WRDI : std_logic_vector(7 downto 0) := "00100000";
constant WRSQ : std_logic_vector(7 downto 0) := "01000000";
constant RDSQ : std_logic_vector(7 downto 0) := "11000000";
constant WPEN : std_logic_vector(7 downto 0) := "00000001";


type COMSTATE is (IDLE, WRITEENA, WRITEENAWAIT, WRITEDIS, EPWRITE, EPWRITENEXT, EPREAD, EPREADNEXT);
signal comstat : COMSTATE := IDLE;
signal rdvalb : std_logic_vector(7 downto 0) := x"00";
signal rdbk : std_logic_vector(0 downto 0) := "0";
signal rdx  : std_logic_vector(4 downto 0) := "00000";
signal rdbt : std_logic_vector(2 downto 0) := "000";
signal epwait : std_logic_vector(11 downto 0) := x"000";
signal rdena : std_logic := '0';

begin

-- ECS <= '0' Chip select on, '1' = off
rdval <= rdvalb;
rdvaladdr <= rdbk & rdx;

  process(clk)
  variable x : integer range 0 to 7;
  begin
  if(clk'event and clk = '1') then
    x := conv_integer(rdbt);
    if(rdena = '1') then
      rdvalb(x) <= EDI;
    end if;
  end if;
  end process;
  
-- process for CS and Data to EEPROM
  process(clk)
  variable nw : integer := 0;
  variable bk : integer;
  variable bt : integer range 0 to 8 := 0;
  variable nx : integer range 0 to 63 := 0;
  begin
  if(clk'event and clk = '0') then
    case comstat is
    when IDLE =>
      ECS <= '1';
      EDO <= '0';
      bk := 0;
      nx := 0;
      bt := 0;
      nw := 0;
      rdwork <= '0';
      rdbk <= "0";
      rdx <= "11111";
      rdvalf <= '0';
      rdena <= '0';
      epwait <= x"000";
      if(wrf = '1') then
        comstat <= WRITEENA;
      elsif(rdf = '1') then
        comstat <= EPREAD;
      else
        comstat <= IDLE;
      end if;
    when WRITEENA =>
      rdvalf <= '0';
      rdena <= '0';
      if(nw < 8) then
        ECS <= '0';
        EDO <= WREN(nw);
        nw := nw + 1;
        comstat <= WRITEENA;
      else
        ECS <= '1';
        EDO <= '1';
        nw := 0;
        comstat <= WRITEENAWAIT;
      end if;
    when WRITEENAWAIT => 
      ECS <= '1';
      EDO <= '0';
      nw := 0;
      comstat <= EPWRITE;
    when WRITEDIS =>
      if(nw < 8) then
        ECS <= '0';
        EDO <= WRDI(nw);
        nw := nw + 1;
      else
        ECS <= '1';
        EDO <= '0';
        nw := 0;
        if(bk = 2) then
          comstat <= IDLE;
          bk := 0;
        else
          comstat <= WRITEENA;
          bt := 0;
        end if;
      end if;
    when EPWRITE =>
      if(nw < 8) then
        ECS <= '0';
        EDO <= WRSQ(nw);
        nw := nw + 1;
        comstat <= EPWRITE;
      elsif(nw < 24) then
        ECS <= '0';
        EDO <= ADDR(bk)(nw - 8);
        nw := nw + 1;
        bt := 7;
        if(bk = 0) then
          nx := 0;
        else
          nx := 32;
        end if;
        comstat <= EPWRITE;
      elsif(nw < 280) then
        ECS <= '0';
        EDO <= wrval(nx)(bt);
        nw := nw + 1;
        if(bt = 0) then
          nx := nx + 1;
          bt := 7;
        else
          bt := bt - 1;
        end if;
        comstat <= EPWRITE;
      else
        ECS <= '1';
        EDO <= '0';
        nw := 0;
        bk := bk + 1;
        comstat <= EPWRITENEXT;
      end if;
    when EPWRITENEXT =>
      if(epwait < 3500) then
        comstat <= EPWRITENEXT;
        epwait <= epwait + '1';
      else
        comstat <= WRITEDIS;
        epwait <= epwait;
      end if;
      ECS <= '1';
      EDO <= '0';
      nw := 0;
    when EPREAD =>
      rdwork <= '1';
      if(nw < 8) then
        ECS <= '0';
        rdena <= '0';
        EDO <= RDSQ(nw);
        nw := nw + 1;
      elsif(nw < 24) then
        ECS <= '0';
        EDO <= ADDR(bk)(nw - 8);
        nw := nw + 1;
      elsif(nw < 280) then
        ECS <= '0';
        rdena <= '1';
        bt := bt + 1;

        if(nw = 24) then
          rdvalf <= '0';
        else
          if(bt = 1) then
            rdvalf <= '1';
          else
            rdvalf <= '0';
          end if;
        end if;

        if(bt = 1) then
          rdbt <= "111";
        elsif(bt = 8) then
          bt := 0;
          rdx <= rdx + '1';
          rdbt <= rdbt - '1';
        else
          rdbt <= rdbt - '1';
          rdvalf <= '0';
        end if;
        comstat <= EPREAD;
        nw := nw + 1;
      else
        ECS <= '1';
        rdena <= '0';
        rdvalf <= '1';
        EDO <= '0';
        nw := 0;
        bk := bk + 1;
        rdbk <= rdbk + '1';
        comstat <= EPREADNEXT;
      end if;
    when EPREADNEXT =>
      rdvalf <= '0';
      rdena <= '0';
      EDO <= '0';
      ECS <= '1';
      nw := 0;
      if(bk = 2) then
        comstat <= IDLE;
      else
        comstat <= EPREAD;
      end if;
    end case;
  end if;
  end process;

end Behavioral;

